@@ -11,6 +11,7 @@
 # under the License.
 
 from collections import OrderedDict
+import hashlib
 import os
 import re
 
@@ -19,6 +20,8 @@ from docutils.parsers import rst
 from docutils.parsers.rst.directives.tables import Table
 from docutils.statemachine import ViewList
 import pbr.version
+import six
+from sphinx.util import logging
 from sphinx.util.osutil import copyfile
 import yaml
 
@@ -29,9 +32,7 @@ from os_api_ref.http_codes import HTTPResponseCodeDirective
 __version__ = pbr.version.VersionInfo(
     'os_api_ref').version_string()
 
-# This is to allow for a graceful swap from oslosphinx to openstackdocstheme
-THEME = 'openstackdocstheme'
-
+LOG = logging.getLogger(__name__)
 
 """This provides a sphinx extension able to create the HTML needed
 for the api-ref website.
@@ -166,8 +167,6 @@ class RestMethodDirective(rst.Directive):
         lineno = self.state_machine.abs_line_number()
         target = nodes.target()
         section = nodes.section(classes=["detail-control"])
-        # env = self.state.document.settings.env
-        # env.app.info("Parent %s" % self.state.parent.attributes)
 
         node = rest_method()
 
@@ -196,7 +195,11 @@ class RestMethodDirective(rst.Directive):
 
         # We need to build a temporary target that we can replace
         # later in the processing to get the TOC to resolve correctly.
-        temp_target = "%s-selector" % node['target']
+        # SHA-1 is used even if collisions are possible, because
+        # they are still unlikely to occurr and it is way shorter
+        # than stronger SHAs.
+        node_hash = hashlib.sha1(str(node).encode('utf-8')).hexdigest()
+        temp_target = "%s-%s-selector" % (node['target'], node_hash)
         target = nodes.target(ids=[temp_target])
         self.state.add_target(temp_target, '', target, lineno)
         section += node
@@ -218,25 +221,22 @@ class RestParametersDirective(Table):
             return YAML_CACHE[fpath]
 
         lookup = {}
-        # self.app.info("Fpath: %s" % fpath)
         try:
             with open(fpath, 'r') as stream:
                 lookup = ordered_load(stream)
         except IOError:
-            self.app.warn(
-                "Parameters file %s not found" % fpath,
-                (self.env.docname, None))
+            LOG.warning("Parameters file not found, %s", fpath,
+                        location=(self.env.docname, None))
             return
         except yaml.YAMLError as exc:
-            self.app.warn(exc)
+            LOG.exception(exc_info=exc)
             raise
 
         if lookup:
             self._check_yaml_sorting(fpath, lookup)
         else:
-            self.app.warn(
-                "Parameters file is empty %s" % fpath,
-                (self.env.docname, None))
+            LOG.warning("Parameters file is empty, %s", fpath,
+                        location=(self.env.docname, None))
             return
 
         YAML_CACHE[fpath] = lookup
@@ -257,15 +257,18 @@ class RestParametersDirective(Table):
 
         last = None
         for key, value in yaml_data.items():
+            if not isinstance(value, dict):
+                raise Exception('Expected a dict for {0}; got {0}={1}).\n'
+                                'You probably have indentation typo in your'
+                                'YAML source'.format(key, value))
+
             # use of an invalid 'in' value
             if value['in'] not in sections:
-                self.app.warn(
-                    "``%s`` is not a valid value for 'in' (must be one of: %s)"
-                    ". (see ``%s``)" % (
-                        value['in'],
-                        ", ".join(sorted(sections.keys())),
-                        key)
-                    )
+                LOG.warning("``%s`` is not a valid value for 'in' (must be "
+                            "one of: %s). (see ``%s``)",
+                            value['in'],
+                            ", ".join(sorted(sections.keys())),
+                            key)
                 continue
 
             if last is None:
@@ -275,17 +278,13 @@ class RestParametersDirective(Table):
             current_section = value['in']
             last_section = last[1]['in']
             if sections[current_section] < sections[last_section]:
-                self.app.warn(
-                    "Section out of order. All parameters in section ``%s`` "
-                    "should be after section ``%s``. (see ``%s``)" % (
-                        last_section,
-                        current_section,
-                        last[0]))
+                LOG.warning("Section out of order. All parameters in section "
+                            "``%s`` should be after section ``%s``. (see "
+                            "``%s``)", last_section, current_section, last[0])
             if (sections[value['in']] == sections[last[1]['in']] and
                 key.lower() < last[0].lower()):
-                self.app.warn(
-                    "Parameters out of order ``%s`` should be after ``%s``" % (
-                        last[0], key))
+                LOG.warning("Parameters out of order ``%s`` should be after "
+                            "``%s``", last[0], key)
             last = (key, value)
 
     def yaml_from_file(self, fpath):
@@ -301,17 +300,14 @@ class RestParametersDirective(Table):
 
         content = "\n".join(self.content)
         parsed = yaml.safe_load(content)
-        # self.app.info("Params loaded is %s" % parsed)
-        # self.app.info("Lookup table looks like %s" % lookup)
         new_content = list()
         for paramlist in parsed:
             if not isinstance(paramlist, dict):
-                self.app.warn(
-                    ("Invalid parameter definition ``%s``. Expected "
-                     "format: ``name: reference``. "
-                     " Skipping." % paramlist),
-                    (self.state_machine.node.source,
-                     self.state_machine.node.line))
+                location = (self.state_machine.node.source,
+                            self.state_machine.node.line)
+                LOG.warning("Invalid parameter definition ``%s``. Expected "
+                            "format: ``name: reference``. Skipping.",
+                            paramlist, location=location)
                 continue
             for name, ref in paramlist.items():
                 if ref in lookup:
@@ -322,11 +318,10 @@ class RestParametersDirective(Table):
                     # used this way, however it does provide a way to
                     # track down where the parameters list is that is
                     # wrong. So it's good enough for now.
-                    self.app.warn(
-                        ("No field definition for ``%s`` found in ``%s``. "
-                         " Skipping." % (ref, fpath)),
-                        (self.state_machine.node.source,
-                         self.state_machine.node.line))
+                    location = (self.state_machine.node.source,
+                                self.state_machine.node.line)
+                    LOG.warning("No field definition for ``%s`` found in "
+                                "``%s``. Skipping.", ref, fpath)
 
                 # Check for path params in stanza
                 for i, param in enumerate(self.env.path_params):
@@ -340,19 +335,15 @@ class RestParametersDirective(Table):
             # Warn that path parameters are not set in rest_parameter
             # stanza and will not appear in the generated table.
             for param in self.env.path_params:
-                self.app.warn(
-                    ("No path parameter ``%s`` found in rest_parameter"
-                     " stanza.\n"
-                     % param.rstrip('}').lstrip('{')),
-                    (self.state_machine.node.source,
-                     self.state_machine.node.line))
+                location = (self.state_machine.node.source,
+                            self.state_machine.node.line)
+                LOG.warning("No path parameter ``%s`` found in rest_parameter"
+                            " stanza.\n", param.rstrip('}').lstrip('{'))
 
-        # self.app.info("New content %s" % new_content)
         self.yaml = new_content
 
     def run(self):
         self.env = self.state.document.settings.env
-        self.app = self.env.app
 
         # Make sure we have some content, which should be yaml that
         # defines some parameters.
@@ -389,7 +380,6 @@ class RestParametersDirective(Table):
             self.col_widths = self.col_widths[1]
         # Actually convert the yaml
         title, messages = self.make_title()
-        # self.app.info("Title %s, messages %s" % (title, messages))
         table_node = self.build_table()
         self.add_name(table_node)
         if title:
@@ -401,7 +391,7 @@ class RestParametersDirective(Table):
         groups = []
         trow = nodes.row()
         entry = nodes.entry()
-        para = nodes.paragraph(text=unicode(table_data))
+        para = nodes.paragraph(text=six.u(table_data))
         entry += para
         trow += entry
         rows.append(trow)
@@ -429,7 +419,6 @@ class RestParametersDirective(Table):
         rows = []
         groups = []
         try:
-            # self.app.info("Parsed content is: %s" % self.yaml)
             for key, values in self.yaml:
                 min_version = values.get('min_version', '')
                 max_version = values.get('max_version', '')
@@ -457,8 +446,8 @@ class RestParametersDirective(Table):
                 rows.append(trow)
         except AttributeError as exc:
             if 'key' in locals():
-                self.app.warn("Failure on key: %s, values: %s. %s" %
-                              (key, values, exc))
+                LOG.warning("Failure on key: %s, values: %s. %s",
+                            key, values, exc)
             else:
                 rows.append(self.show_no_yaml_error())
         return rows, groups
@@ -649,10 +638,11 @@ def copy_assets(app, exception):
         'glyphicons-halflings-regular.ttf',
         'glyphicons-halflings-regular.woff'
     )
-    if app.builder.name != 'html' or exception:
+    builders = ('html', 'readthedocs', 'readthedocssinglehtmllocalmedia')
+    if app.builder.name not in builders or exception:
         return
-    app.info('Copying assets: %s' % ', '.join(assets))
-    app.info('Copying fonts: %s' % ', '.join(fonts))
+    LOG.info('Copying assets: %s', ', '.join(assets))
+    LOG.info('Copying fonts: %s', ', '.join(fonts))
     for asset in assets:
         dest = os.path.join(app.builder.outdir, '_static', asset)
         source = os.path.abspath(os.path.dirname(__file__))
@@ -677,12 +667,9 @@ def setup(app):
     # TODO(sdague): if someone wants to support latex/pdf, or man page
     # generation using these stanzas, here is where you'd need to
     # specify content specific renderers.
-    app.add_node(rest_method,
-                 html=(rest_method_html, None))
-    app.add_node(rest_expand_all,
-                 html=(rest_expand_all_html, None))
-    app.add_node(http_code,
-                 html=(http_code_html, None))
+    app.add_node(rest_method, html=(rest_method_html, None))
+    app.add_node(rest_expand_all, html=(rest_expand_all_html, None))
+    app.add_node(http_code, html=(http_code_html, None))
 
     # This specifies all our directives that we're adding
     app.add_directive('rest_parameters', RestParametersDirective)
